package com.ruoyi.common;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.domain.*;
import com.ruoyi.mapper.TaskMapper;
import io.swagger.models.auth.In;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
@Component
public class DataStatistics extends ServiceImpl<TaskMapper, Task> {

    public int calculateStandards(List<ReviewStandard> standards) {
        return standards.size();
    }

    public int calculateStandardsPass(List<ReviewStandard> standards, Map<Long, ReviewDetailsResult> resultMap) {

        int standardsPass = 0;

        for(ReviewStandard standard : standards) {

            List<ReviewDetails> details = getDetailsByStandard(standard);

            boolean isPass = false;

            for(ReviewDetails reviewDetails : details) {

                Integer passed = detailsIsPass(reviewDetails, resultMap);
                if(passed == null) {
                    isPass = false;
                }

                if(passed == ReviewStandard.NO_PASSED) {
                    isPass = false;
                }

//                isPass = passed == ReviewStandard.PASSED;
            }

            if(isPass) {
                standardsPass = standardsPass + 1;
            }
        }

        return standardsPass;

    }

    public int calculateStandardsNoPass(List<ReviewStandard> standards, Map<Long, ReviewDetailsResult> resultMap) {

        int standardsNoPass = 0;

        for(ReviewStandard standard : standards) {

            List<ReviewDetails> details = getDetailsByStandard(standard);
            boolean isPass = false;

            for(ReviewDetails reviewDetails : details) {

                Integer passed = detailsIsPass(reviewDetails, resultMap);
                if(passed == null) {
                    isPass = false;
                }

                if(passed == ReviewStandard.PASSED) {
                    isPass = false;
                }


//                isPass = passed == ReviewStandard.NO_PASSED;
            }

            if(isPass) {
                standardsNoPass = standardsNoPass + 1;
            }
        }

        return standardsNoPass;

    }

    public int calculateDetails(List<ReviewStandard> standards) {

        //审查细则(条)
        int detailsNum = 0;

        for(ReviewStandard standard : standards) {

            List<ReviewDetails> details = getDetailsByStandard(standard);
            detailsNum = detailsNum + details.size();

        }

        return detailsNum;

    }

    public int calculateDetailsPass(List<ReviewStandard> standards, Map<Long, ReviewDetailsResult> resultMap) {

        int detailsPass = 0;

        for(ReviewStandard standard : standards) {
            List<ReviewDetails> details = getDetailsByStandard(standard);
            for(ReviewDetails reviewDetails : details) {

                if (detailsIsPass(reviewDetails, resultMap) == null) {
                    continue;
                }

                if(detailsIsPass(reviewDetails, resultMap) == ReviewStandard.PASSED) {
                    detailsPass = detailsPass + 1;
                }
            }
        }

        return detailsPass;

    }

    public int calculateDetailsNoPass(List<ReviewStandard> standards, Map<Long, ReviewDetailsResult> resultMap) {

        int detailsNoPass = 0;

        for(ReviewStandard standard : standards) {
            List<ReviewDetails> details = getDetailsByStandard(standard);
            for(ReviewDetails reviewDetails : details) {

                if(detailsIsPass(reviewDetails, resultMap) == ReviewStandard.NO_PASSED) {
                    detailsNoPass = detailsNoPass + 1;
                }
            }
        }

        return detailsNoPass;

    }

    public List<ReviewDetails> getDetailsByStandard(ReviewStandard standard) {

        List<ReviewDetails> details = new ArrayList();

        List<ReviewKeyPoint> keyPointList = standard.getKeyPointList();
        for(ReviewKeyPoint keyPoint : keyPointList) {
            details.addAll(keyPoint.getReviewDetailsList());
        }

        return details;
    }

    public Integer detailsIsPass(ReviewDetails reviewDetails, Map<Long, ReviewDetailsResult> resultMap) {
        ReviewDetailsResult result = resultMap.get(reviewDetails.getId());

        if(result == null) {
            return null;
        }

        return result.getPassed();
    }
}
